//+------------------------------------------------------------------+
//|                                               smHullSlope v5.mq4 |
//|                                            2009.07.06, SwingMan |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " 2009.07.06, SwingMan"
#property link      ""

#property indicator_separate_window

#property indicator_buffers 6
#property indicator_color1 Gray
#property indicator_color2 LimeGreen
#property indicator_color3 RoyalBlue
#property indicator_color4 LimeGreen
#property indicator_color5 RoyalBlue
#property indicator_color6 Gold

#property indicator_width1 1 // histogram
#property indicator_width2 3
#property indicator_width3 3
#property indicator_width4 3 // zero line
#property indicator_width5 3
#property indicator_width6 1 // line


#property indicator_level1 0

//---- inputs --------------------------------------------------------
extern int Avg_period= 27;
extern int smooth_period  = 5;
extern double treshold = 10;
extern int nBars = 1000;
//--------------------------------------------------------------------

double Line1[], Line2[];
double HullVector[], HullBuffer[];
double TrendUP[], TrendDN[];
double HistoUP[], HistoDN[], HistoNN[];

double Avg_periodDivisor = 1.5;
int Avg_method = MODE_LWMA;
int Avg_price  = PRICE_CLOSE;
int Avg_shift = 1;

//---------------------------------------------------------------------------------|
//                                                                                 |
//---------------------------------------------------------------------------------|
int init()
{
int iArrow = 167;//159;//167;
   ArraySetAsSeries(HullVector, true);
   ArraySetAsSeries(HullBuffer, true);
   ArraySetAsSeries(Line2, true);
   
   SetIndexBuffer(0,HistoNN); SetIndexLabel(0,"trend");      SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,HistoUP); SetIndexLabel(1,"retrace UP"); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,HistoDN); SetIndexLabel(2,"retrace DN"); SetIndexStyle(2,DRAW_HISTOGRAM);   

   SetIndexBuffer(3,TrendUP); SetIndexLabel(3,"sig UP"); SetIndexStyle(3,DRAW_ARROW); SetIndexArrow(3,iArrow);
   SetIndexBuffer(4,TrendDN); SetIndexLabel(4,"sig DN"); SetIndexStyle(4,DRAW_ARROW); SetIndexArrow(4,iArrow);
      
   SetIndexBuffer(5,Line1); SetIndexLabel(5,"slope"); SetIndexStyle(5,DRAW_LINE);
   
   IndicatorShortName("smHullSlope v5 ("+Avg_period+")");
   return(0);
}

//---------------------------------------------------------------------------------|
//                                                                                 |
//---------------------------------------------------------------------------------|
int deinit()
{
   return(0);
}

//---------------------------------------------------------------------------------|
//                                                                                 |
//---------------------------------------------------------------------------------|
int start() 
{ 
   ArrayResize(HullVector, Bars);
   ArrayResize(HullBuffer, Bars);
   ArrayResize(Line2, Bars);
   
string sSymbol = Symbol();
int iTF = Period();
int iTrend;
int p = MathFloor(MathSqrt(Avg_period));
int perVector = MathFloor(Avg_period/Avg_periodDivisor);   

   //int counted_bars = IndicatorCounted();
   int i, limit;
   int limit2 = nBars;
   
   //if (counted_bars < 0) return(-1);
   //if (counted_bars > 0) counted_bars--;
   //limit=Bars - counted_bars;
  
   //---- HULL vector
   for (i=limit2;i>=0;i--)
      HullVector[i] = 2*WMA(sSymbol, iTF, i, perVector, Avg_method, Avg_price) -
                  WMA(sSymbol, iTF, i, Avg_period, Avg_method, Avg_price);
   
   //---- HULL buffer
   for (i=limit2;i>=0;i--)
      HullBuffer[i] = iMAOnArray(HullVector, 0, p, 0, Avg_method, i);

   //################################################################   
   //---- MAvg slope
   for (i=limit2;i>=0;i--)
   {        
      if (HullBuffer[i+1] != 0)
         Line1[i] = 100*(HullBuffer[i] - HullBuffer[i+1]) / HullBuffer[i+1];
   }
   
   for (i=limit2;i>=0;i--)
   {        
      //-- Trigger line
      Line2[i] = (4.0*Line1[i] + 3.0*Line1[i+1] + 2.0*Line1[i+2] + Line1[i+3]) / 10.0;
      
      double minVal = MathMin(Line1[i], Line2[i]);
      double maxVal = MathMax(Line1[i], Line2[i]);
      if (minVal != 0)
         double diffP = MathAbs(100 * (maxVal - minVal) / minVal);

      if (Line1[i] > Line2[i]) {
         if (diffP > treshold) {
            TrendUP[i] = 0;
            TrendDN[i] = EMPTY_VALUE;
         }
         else{
            TrendUP[i] = TrendUP[i+1];
            TrendDN[i] = TrendDN[i+1];
         }
      }
      else
      if (Line1[i] < Line2[i]) {
         if (diffP > treshold) {
            TrendUP[i] = EMPTY_VALUE;
            TrendDN[i] = 0;
         }
         else{
            TrendUP[i] = TrendUP[i+1];
            TrendDN[i] = TrendDN[i+1];  
         }
      }
      
      //-- long trend
      if (Line1[i] > 0 && TrendUP[i] == 0) {
         if (High[i] <= High[i+1]) {
            HistoUP[i] = Line1[i];
            HistoDN[i] = EMPTY_VALUE;
            HistoNN[i] = EMPTY_VALUE;
         }
         else {
            HistoUP[i] = EMPTY_VALUE;
            HistoDN[i] = EMPTY_VALUE;
            HistoNN[i] = Line1[i];
         }
      }
      else
      //-- short trend
      if (Line1[i] < 0 && TrendDN[i] == 0) {
         if (Low[i] >= Low[i+1]) {
            HistoUP[i] = EMPTY_VALUE;
            HistoDN[i] = Line1[i]; 
            HistoNN[i] = EMPTY_VALUE;
         }
         else {
            HistoUP[i] = EMPTY_VALUE;
            HistoDN[i] = EMPTY_VALUE; 
            HistoNN[i] = Line1[i];
         }
      }
   }
   return(0);
}

//+------------------------------------------------------------------+ 
//|      WMA
//+------------------------------------------------------------------+ 
double WMA(string sSymbol, int iTF, int iBar, 
           int iPeriodAvg, int Hull_method, int Hull_price) 
{ 
   return(iMA(sSymbol, iTF, iPeriodAvg, 0, Hull_method, Hull_price, iBar));
} 

